@@ -12,14 +12,7 @@ import yt_dlp
 
 from xklb.db import sqlite_con
 from xklb.tube_actions import default_ydl_opts
-from xklb.utils import (
-    argparse_dict,
-    combine,
-    filter_None,
-    log,
-    safe_unpack,
-    single_column_tolist,
-)
+from xklb.utils import argparse_dict, combine, filter_None, log, safe_unpack, single_column_tolist
 
 
 def parse_args(action):
@@ -28,7 +21,6 @@ def parse_args(action):
     parser.add_argument("--db", "-db")
     if action == "add":
         parser.add_argument("playlists", nargs="+")
-        parser.add_argument("-f", "--overwrite-db", action="store_true", help="Delete db file before scanning")
         parser.add_argument(
             "--lightweight",
             "-lw",
@@ -57,10 +49,7 @@ def parse_args(action):
     if args.db:
         args.database = args.db
 
-    if args.action == "add":
-        if args.overwrite_db:
-            Path(args.database).unlink(missing_ok=True)
-        Path(args.database).touch()
+    Path(args.database).touch()
     args.con = sqlite_con(args.database)
 
     ydl_opts = {**default_ydl_opts, **args.yt_dlp_config}
@@ -168,6 +157,7 @@ def consolidate(playlist_path, v):
     duration = v.pop("duration", None)
     cv["duration"] = 0 if not duration else int(duration)
     cv["play_count"] = 0
+    cv["time_played"] = None
     cv["language"] = v.pop("language", None)
     cv["tags"] = combine(v.pop("description", None), v.pop("categories", None), v.pop("tags", None))
     cv["id"] = v.pop("id")
